--[[
	TODO:
	- Hide any running bars that are now hidden when the Spell enable/disable options are changed
	- Color bars by player class (needs a LibBars-1.0 change)
	- Change bar height in real time (needs a LibBars-1.0 change)
]]

local MINOR_VERSION = tonumber(("$Revision: 71020 $"):match("%d+"))
local AceConfig = LibStub("AceConfigDialog-3.0")
local Media = LibStub("LibSharedMedia-3.0")
--local L = LibStub("AceLocale-3.0"):GetLocale("RaidCooldowns")

RCD_Display = LibStub("AceAddon-3.0"):NewAddon("RaidCooldowns_Display", "AceComm-3.0", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")
RCD_Display.MINOR_VERSION = MINOR_VERSION

local barGroup = nil

local defaults = {
	profile = {
		locked   = false,
		scale    = 100.0,
		texture  = "Smooth v2",
		growUp   = false,
		fontFace = nil,
		fontSize = 10,
		
		width  = 125,
		height = 10,
		
		spells = { },
	}
}
local optFrame
function RCD_Display:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RCDD_DB", nil, "Default")
	
	if not self.db.profile.spells or #self.db.profile.spells == 0 then
		-- Default all spells to enabled
		for k,v in pairs(self.cooldowns) do
			for k2, v2 in pairs(v) do
				defaults.profile.spells[v2] = true
			end
		end
	end
	
	self.db:RegisterDefaults(defaults)
	
	optFrame = AceConfig:AddToBlizOptions("RaidCooldowns", "RaidCooldowns")
	
	self:UnregisterAllEvents()
	self:RegisterEvent("PLAYER_LOGIN")

	self.prefix = "RaCD"
	self.userAgent = "RaidCooldowns"
	self:RegisterComm(self.prefix)
	
	self:CreateFrame()
end

function RCD_Display:OnEnable()
end

function RCD_Display:OnDisable()
end

function RCD_Display:OnCommReceived(prefix, msg, distro, sender)
	local spellId, cooldown = select(3, msg:find("(%d+) (%d+)"))
	spellId  = tonumber(spellId)
	cooldown = tonumber(cooldown)
	
	-- Only show a bar for this spell if the user enabled it
	if self.db.profile.spells[spellId] then
		barGroup:NewTimerBar(self:NameBar(sender, spellId), sender, cooldown, cooldown, spellId)
	end
end

function RCD_Display:PLAYER_LOGIN()
	self:SetAnchors(true)
	self:UpdateDisplay()
	self:UnregisterEvent("PLAYER_LOGIN")
end

--[[		Configuration Methods		]]--

function RCD_Display:ShowConfig()
	AceConfig:SetDefaultSize("RaidCooldowns", 500, 550)
	AceConfig:Open("RaidCooldowns", configFrame)
end

function RCD_Display:Test()
	self = RCD_Display
	
	local playerName = UnitName("player")
	
	local i = 1
	for _, v in pairs(self.cooldowns) do
		for _, v2 in pairs(v) do
			self:OnCommReceived(self.prefix, (v2 .. " " .. i * 10), "RAID", playerName)
			i = i + 1
		end
	end
end

--[[		Bar Group Anchor		]]--

function RCD_Display:CreateFrame()
	barGroup = nil
	
	barGroup = self:NewBarGroup("RaidCooldowns", nil, self.db.profile.width, self.db.profile.height, "RCDD_Anchor")
	barGroup:SetClampedToScreen(true)
	barGroup:SetColorAt(1.00, 1, 0, 0, 1)
	barGroup:SetColorAt(0.66, 1, 1, 0, 1)
	barGroup:SetColorAt(0.33, 0, 1, 1, 1)
	barGroup:SetColorAt(0.00, 0, 0, 1, 1)
	barGroup:SetFlashPeriod(0)
	
	-- Callbacks
	barGroup.RegisterCallback(self, "AnchorClicked")
	barGroup.RegisterCallback(self, "AnchorMoved")
	--barGroup.RegisterCallback(self, "TimerFinished")
end

function RCD_Display:UpdateDisplay()
	if not barGroup then return end
	
	if self.db.profile.locked then
		barGroup:Lock()
		barGroup:HideAnchor()
	else
		barGroup:Unlock()
		barGroup:ShowAnchor()
	end

	barGroup:SetFont(Media:Fetch("font", self.db.profile.fontFace), self.db.profile.fontSize)
	barGroup:SetTexture(Media:Fetch("statusbar", self.db.profile.texture))
	barGroup:SetScale(self.db.profile.scale / 100.0)
	barGroup:ReverseGrowth(self.db.profile.growUp)
end

-- Omen rip
function RCD_Display:SetAnchors(useDB)
	local t = self.db.profile.growUp
	local x, y
	if useDB then
		x, y = self.db.profile.posX, self.db.profile.posY
		if not x and not y then
			barGroup:ClearAllPoints()
			barGroup:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
			return
		end
	elseif t then
		x, y = barGroup:GetLeft(), barGroup:GetBottom()
	else
		x, y = barGroup:GetLeft(), barGroup:GetTop()
	end
	barGroup:ClearAllPoints()
	if t then
		barGroup:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x, y)
	else
		barGroup:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
	end
	self.db.profile.posX, self.db.profile.posY = x, y
end

function RCD_Display:AnchorClicked(cbk, group, button)
	if button == "RightButton" then
		self:ShowConfig()
	end
end

function RCD_Display:AnchorMoved(cbk, group, x, y)
	self:SetAnchors()
end

function RCD_Display:TimerFinished(cbk, group, bar, name)
end

--[[		Something		]]--

function RCD_Display:NameBar(sender, spell)
	return sender .. "_" .. tostring(spell)
end